/* Copyright (c) 2004 Marijn Haverbeke
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any
 * damages arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any
 * purpose, including commercial applications, and to alter it and
 * redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software. If you use this
 *    software in a product, an acknowledgment in the product
 *    documentation would be appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must
 *    not be misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 *
 * Marijn Haverbeke
 * marijn(at)haverbeke.nl
 */

#ifndef CANVAS_HPP
#define CANVAS_HPP

#include <wx/wx.h>

class Jigsaw;
class Piece;

// The canvas shows the puzzle pieces and handles dragging them
// around.
class Canvas: public wxWindow
{
public:
  Canvas(wxWindow* parent);
  ~Canvas();
  
  void Paint(wxPaintEvent& event);
  void Resize(wxSizeEvent& event);
  
  void Mouse_Down(wxMouseEvent& event);
  void Mouse_Up(wxMouseEvent& event);
  void Mouse_Move(wxMouseEvent& event);

  void Start_Clear_Area(wxMouseEvent& event);
  void Clear_Area(wxMouseEvent& event);

  void Set_Game(Jigsaw* game);
  void Set_Background(const wxColor& color);
  void Set_Background(const wxImage& image);
  
private:
  void Repaint_Back_Buffer();
  void Repaint_Back_Buffer(const wxRect& area);
  void Tile_Back_Image(wxDC& dc, const wxRect& area);
  
  void Draw_Completed_Message(wxDC& dc);
  void Move_Dragged_Block(const wxPoint& distance);
  void Draw_Clear_Area(const wxPoint& mouse);

  Jigsaw* _game;
  Piece* _dragging;
  wxPoint _drag_offset;
  wxPoint _clear_drag_start;

  wxBitmap _back_buffer;
  wxBitmap* _background;
  wxColor _back_color;

  bool _completed;
  wxFont _completed_font;
  
  DECLARE_EVENT_TABLE();
};

#endif //CANVAS_HPP
