/* Copyright (c) 2004 Marijn Haverbeke
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any
 * damages arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any
 * purpose, including commercial applications, and to alter it and
 * redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software. If you use this
 *    software in a product, an acknowledgment in the product
 *    documentation would be appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must
 *    not be misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 *
 * Marijn Haverbeke
 * marijn(at)haverbeke.nl
 */

#ifndef CUTTER_HPP
#define CUTTER_HPP

#include <wx/wx.h>
#include <vector>

class Piece;

// This class generates the shapes for the puzzle and adds masks to
// the pieces' images to make them look right.
class Jigsaw_Cutter
{
public:
  Jigsaw_Cutter(size_t width, size_t height);

  void Cut_Piece(Piece* piece);

private:
  // Every side has side id (shape) and a bool indicating whether the
  // thing is mirrored or not.
  struct Side
  {
    Side(unsigned char id, bool reverse)
      : id(id), reverse(reverse) {}
    Side()
      : id(0), reverse(false) {}
    unsigned char id;
    bool reverse;
  };
  // Every piece has 4 sides. This duplicates some information (sides
  // depend on the sides of the pieces next to it), but is much more
  // convenient.
  struct Piece_Shape
  {
    Side top;
    Side bottom;
    Side left;
    Side right;
  };

  void Cut_Corners(wxDC& mask);
  void Cut_Side_Vertical(wxDC& mask, Piece_Shape& shape, bool left);
  void Cut_Side_Horizontal(wxDC& mask, Piece_Shape& shape, bool top);

  std::vector<Piece_Shape> _shapes;

  size_t _width;
  size_t _height;
};

#endif //CUTTER_HPP
